#include "chil_global.h"
char secondary_sum_dir[MAX_FILENAME_LENGTH] = ".";

int
CHILGetFileList(uword day,uword month,uword year,int Argc,char **Argv,
		FileList * filelist)
{
  int i,status, nosumfile = 0;
  glob_t globbuf;
  FILE *sumfile;
  uword raster,yraster=0,pos=0;
  uword tday,tmonth,tyear;
  uword yday,ymonth,yyear;
  unsigned int GLOB_FLAG=0;
  char *lastfilename=NULL;
  char tmp[255];

  filelist->n=0;

#ifdef DEBUG
  CHILPrintf("Globbing the file list\n");
#endif
  /* First we want to glob the argument list (stopping if we reach one 
     that begins with "-") to obtain a list of filenames */
  for (i=0;i<Argc;i++) {
    if (*Argv[i] == '-')
      break;
    if ((status=glob(Argv[i],GLOB_FLAG,NULL,&globbuf)) == GLOB_NOSPACE) {
      globfree(&globbuf);
      CHILPrintf("Out of memory\n");
      return -1;
    }
    GLOB_FLAG=GLOB_APPEND;
  }

#ifdef DEBUG
  CHILPrintf("Getting yesterdays date\n");
#endif
  /* get date for yesterday */
  yday=day;ymonth=month;yyear=year;
  CHILDecrDate(&yday,&ymonth,&yyear);

  /* Make sure that year and yyear are in the range 0-99! */
  if (year > 99) {
    year -= 100;
  }
  if (yyear > 99) {
    yyear -= 100;
  }  

#ifdef DEBUG
  CHILPrintf("Searching summary files\n");
#endif

  status=0;
  /* Loop through all matched filenames, search corresponding 
     summary files for the date required */
  for (i=0;i<globbuf.gl_pathc;i++) {
    if ((sumfile=CHILOpenSummary(globbuf.gl_pathv[i])) != NULL) {
#ifdef DEBUG
      CHILPrintf("Openned %s\n",globbuf.gl_pathv[i]);
#endif
      fgets(tmp,255,sumfile); /* skip first line */
      while(fgets(tmp, 255, sumfile) != '\0') {
	sscanf(tmp, "%*5d %3hu %*8c %2hu/%2hu/%2hu", 
	       &raster, &tday, &tmonth, &tyear);
#ifdef DEBUG
	CHILPrintf("Read a line\n");
#endif
	/* Decide if raster is for yesterday, today or neither */
	if (status==0 && tday==yday && tmonth==ymonth && tyear==yyear) {
	  yraster=raster;
	}
	else if (tday==day && tmonth==month && tyear==year) {
	  if (status==0 && yraster > 0) {
	    strncpy(filelist->name[0]=filelist->buffer, lastfilename,
		    MAX_FILENAME_LENGTH);
	    pos=strlen(lastfilename)+1;
	    filelist->first[0]=filelist->last[0]=yraster;
	    (filelist->n)=1;
	    status=1;
	    yraster=0;
	  }
	  if (lastfilename != globbuf.gl_pathv[i]) {
	    if (filelist->n >= MAX_FILES) {
	      status=2;
	      break;
	    }
	    strncpy(filelist->name[filelist->n]=filelist->buffer+pos,
		    globbuf.gl_pathv[i], MAX_FILENAME_LENGTH);
	    filelist->first[filelist->n]=filelist->last[filelist->n]=raster;
	    pos+=strlen(globbuf.gl_pathv[i])+1;
	    ++filelist->n;
	  }
	  else {
	    filelist->last[filelist->n-1]=raster;
	  }
	}
	else if (status==1) {
	  if (yraster > 0) {
	    strncpy(filelist->name[0]=filelist->buffer, lastfilename,
		    MAX_FILENAME_LENGTH);
	    filelist->first[0]=filelist->last[0]=yraster;
	    (filelist->n)=1;
	  }
	  status=2;
	  break;
	}
	lastfilename=globbuf.gl_pathv[i];
      }
      fclose(sumfile);
      if (status==2)
	break;
    }
    else {
      nosumfile = 1;
    }
  }
#ifdef DEBUG
  CHILPrintf("Freeing list\n");
#endif
  globfree(&globbuf);
  if (nosumfile) {
    CHILPrintf("One or more data files have no corresponding summary file\n");
  }
  return filelist->n;
}

FILE *
CHILOpenSummary(char * datfilename)
{
  char sumfilename[MAX_FILENAME_LENGTH];
  char *dot,*slash;
  FILE *sumfile=NULL;

  strcpy(sumfilename,datfilename);
  if ((dot=strrchr(sumfilename,'.')) != '\0') {
    if (strncmp(dot+1,"dat",3) == 0 || strncmp(dot+1,"DAT",3) == 0) {
      /* If filename is .DAT, look for .SUM */
      if (dot[1] == 'D') {
	strncpy(dot+1,"SUM",3);
      }
      else {
	strncpy(dot+1,"sum",3);
      }
     if ((sumfile=fopen(sumfilename,"r")) == NULL) {	
	/* Summary file not found in the same directory as the data... 
	   try contents of secondary_sum_dir */
	if (secondary_sum_dir[0]=='\0')
	  return NULL;
	strncpy(sumfilename,secondary_sum_dir,MAX_FILENAME_LENGTH);
	if ((slash=strrchr(datfilename,'/')) !='\0') {
	  strcpy(sumfilename+strlen(sumfilename),slash);
	}
	else {
	  strcpy(sumfilename+strlen(sumfilename),"/");
	  strcpy(sumfilename+strlen(sumfilename),datfilename);
	}
	if ((dot=strrchr(sumfilename,'.')) != '\0') {
	  if (strncmp(dot+1,"dat",3) == 0 || strncmp(dot+1,"DAT",3) == 0) {
	    /* If filename is .DAT, look for .SUM */
	    if (dot[1] == 'D') {
	      strncpy(dot+1,"SUM",3);
	    }
	    else {
	      strncpy(dot+1,"sum",3);
	    }
	    return (fopen(sumfilename,"r"));
	  }
	}
      }
    }
  }
  return sumfile;
}

char *
CHILSecondarySumDir(char * sec_sum_dir)
{
  if (strlen(sec_sum_dir) >= MAX_FILENAME_LENGTH-1) {
    CHILPrintf("Warning: directory name %s is too long\n", sec_sum_dir);
    return NULL;
  }
  return (strncpy(secondary_sum_dir,sec_sum_dir,MAX_FILENAME_LENGTH));
}

